using System.ComponentModel;
using System.Linq;

namespace WindowsFormsApplication1.Entities
{
    public class EntityBase : IEntityErrorInfo
    {
        private EntityErrorCollection _errorInfos = new EntityErrorCollection();

        string IDataErrorInfo.this[string propertyName]
        {
            get
            {
                var propertyError = _errorInfos.PropertyErrors.FirstOrDefault(x => x.PropertyName == propertyName);
                return propertyError == null ? string.Empty : propertyError.Error;
            }
        }

        string IDataErrorInfo.Error
        {
            get { return _errorInfos.Error ?? string.Empty; }
        }

        void IEntityErrorInfo.SetError(string propertyName, string message)
        {
            if (string.IsNullOrWhiteSpace(propertyName))
                return;

            //Clear existing errors
            ((IEntityErrorInfo)this).ClearError(propertyName);

            if (!string.IsNullOrWhiteSpace(message))
            {
                _errorInfos.PropertyErrors.Add(new EntityErrorInfo() { PropertyName = propertyName, Error = message });
                _errorInfos.Error = string.Format("{0} has errors", GetType().Name);
            }
        }

        void IEntityErrorInfo.ClearErrors()
        {
            _errorInfos.Error = null;
            _errorInfos.PropertyErrors.Clear();
        }

        void IEntityErrorInfo.ClearError(string propertyName)
        {
            if (string.IsNullOrWhiteSpace(propertyName))
            {
                ((IEntityErrorInfo)this).ClearErrors();
            }
            else
            {
                _errorInfos.PropertyErrors
                           .Where(x => x.PropertyName == propertyName)
                           .ToList()
                           .ForEach(x => _errorInfos.PropertyErrors.Remove(x));

                if (!_errorInfos.PropertyErrors.Any())
                    _errorInfos.Error = null;
            }
        }
    }
}