﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win.UltraWinGrid;
using WindowsFormsApplication1.Entities;
using BeforeExitEditModeEventArgs = Infragistics.Win.UltraWinGrid.BeforeExitEditModeEventArgs;

namespace WindowsFormsApplication1.Views
{
    public partial class MainView : Form
    {
        public MainView()
        {
            InitializeComponent();

            if (LicenseManager.UsageMode != LicenseUsageMode.Designtime)
            {
                SetGridSettings();
                SetGridDataSource();
            }
        }

        #region Grid Initialisation

        private void SetGridSettings()
        {
            ultraGrid1.DisplayLayout.Override.AllowUpdate = DefaultableBoolean.True;
            ultraGrid1.DisplayLayout.Override.SupportDataErrorInfo = SupportDataErrorInfo.RowsAndCells;
            ultraGrid1.DisplayLayout.Override.DataErrorCellAppearance.BackColor = Color.MistyRose;

            ultraGrid1.InitializeLayout += UltraGrid1OnInitializeLayout;
            ultraGrid1.BeforeExitEditMode += UltraGrid1OnBeforeExitEditMode;
        }

        private void SetGridDataSource()
        {
            ultraGrid1.DataSource = new BindingList<CompanyEntity>(GetCompanyEntities(GetCountryEntities()));
        }

        #endregion

        #region DataSource

        private static List<CountryEntity> GetCountryEntities()
        {
            return new List<CountryEntity>()
            {
                new CountryEntity()
                {
                    ID = 1,
                    Code = "BE",
                    Name = "Belgium"
                },
                new CountryEntity()
                {
                    ID = 2,
                    Code = "CN",
                    Name = "China"
                },
                new CountryEntity()
                {
                    ID = 3,
                    Code = "CA",
                    Name = "Canada"
                },
                new CountryEntity()
                {
                    ID = 4,
                    Code = "DK",
                    Name = "Denmark"
                },
                new CountryEntity()
                {
                    ID = 5,
                    Code = "HT",
                    Name = "Haiti"
                },
                new CountryEntity()
                {
                    ID = 6,
                    Code = "NF",
                    Name = "Norfolk Islands"
                },
                new CountryEntity()
                {
                    ID = 7,
                    Code = "SH",
                    Name = "St. Helena"
                },
                new CountryEntity()
                {
                    ID = 8,
                    Code = "VE",
                    Name = "Venezuela"
                }
            };
        }

        private static List<CompanyEntity> GetCompanyEntities(List<CountryEntity> countryEntities)
        {
            return new List<CompanyEntity>()
            {
                new CompanyEntity()
                {
                    ID = 1,
                    Name = "Belgacom",
                    CountryID = countryEntities.Single(x=> x.Code == "BE").ID
                },
                new CompanyEntity()
                {
                    ID = 2,
                    Name = "Microsoft",
                    CountryID = countryEntities.Single(x=> x.Code == "CN").ID
                },
                new CompanyEntity()
                {
                    ID = 3,
                    Name = "IBM",
                    CountryID = countryEntities.Single(x=> x.Code == "CA").ID
                }
            };
        }

        #endregion

        #region Initialize Layout

        private void UltraGrid1OnInitializeLayout(object sender, InitializeLayoutEventArgs initializeLayoutEventArgs)
        {
            var grid = (UltraGrid)sender;
            var columns = initializeLayoutEventArgs.Layout.Bands[0].Columns;
            foreach (var column in columns)
            {
                if (column.Key == "Name")
                {
                    column.CellActivation = Activation.AllowEdit;
                    column.CellClickAction = CellClickAction.EditAndSelectText;
                }
                else if (column.Key == "CountryID")
                {
                    column.CellActivation = Activation.AllowEdit;
                    column.CellClickAction = CellClickAction.EditAndSelectText;

                    var editorSettings = new DefaultEditorOwnerSettings();

                    CountryColumnControl.ValueMember = "ID";
                    CountryColumnControl.DisplayMember = "Name";
                    CountryColumnControl.DataSource = GetCountryEntities();

                    editorSettings.ValueList = CountryColumnControl;

                    editorSettings.DataType = typeof(int);
                    var editor = new EditorWithCombo(new DefaultEditorOwner(editorSettings));
                    column.Editor = editor;
                    column.Style = Infragistics.Win.UltraWinGrid.ColumnStyle.DropDownValidate;

                }
                else
                {
                    column.Hidden = true;
                }

            }
        }

        private UltraDropDown _countryColumnControl;
        public UltraDropDown CountryColumnControl
        {
            get
            {
                if (_countryColumnControl == null)
                {
                    _countryColumnControl = new UltraDropDown();
                    DropDownSettings();
                    _countryColumnControl.InitializeLayout += InitializeLayoutInternal;
                }
                return _countryColumnControl;
            }
        }

        private void InitializeLayoutInternal(object sender, InitializeLayoutEventArgs e)
        {
        }

        private void DropDownSettings()
        {
            CountryColumnControl.DisplayLayout.NewColumnLoadStyle = NewColumnLoadStyle.Show;
            CountryColumnControl.DisplayLayout.ScrollBounds = Infragistics.Win.UltraWinGrid.ScrollBounds.ScrollToFill;
            CountryColumnControl.DisplayLayout.ScrollStyle = Infragistics.Win.UltraWinGrid.ScrollStyle.Immediate;
            CountryColumnControl.DisplayLayout.AutoFitStyle = AutoFitStyle.None;
            CountryColumnControl.DisplayLayout.ViewStyle = ViewStyle.SingleBand;
        }

        #endregion

        #region Events

        private void UltraGrid1OnBeforeExitEditMode(object sender, BeforeExitEditModeEventArgs beforeExitEditModeEventArgs)
        {
            if (ultraGrid1.ActiveCell.Column.Key == "CountryID")
            {
                if (string.IsNullOrWhiteSpace(ultraGrid1.ActiveCell.Text))
                {
                    ((IEntityErrorInfo)ultraGrid1.ActiveCell.Row.ListObject).SetError(ultraGrid1.ActiveCell.Column.Key, "Country is required.");
                }

            }
            else if (ultraGrid1.ActiveCell.Column.Key == "Name")
            {
                if (string.IsNullOrWhiteSpace(ultraGrid1.ActiveCell.Text))
                {
                    ((IEntityErrorInfo)ultraGrid1.ActiveCell.Row.ListObject).SetError(ultraGrid1.ActiveCell.Column.Key, "Name is required.");
                }
            }
        }

        #endregion
    }
}
